@@ -43,7 +43,7 @@
 #include <pwd.h>
 #include <grp.h>
 
-#include <glib-2.0/glib.h>
+#include <glib.h>
 #include <glib/gi18n.h>
 
 #include "vicious.h"
@@ -217,6 +217,7 @@ static gchar *GdmGraphicalThemeColor;
 static gchar *GdmGraphicalThemeDir;
 static gchar *GdmGraphicalThemes;
 static gchar *GdmPreFetchProgram;
+static gchar *GdmUse24Clock;
 
 static gint GdmPositionX;
 static gint GdmPositionY;
@@ -227,6 +228,8 @@ static gint GdmBackgroundType;
 static gint GdmScanTime;
 static gint GdmMaxWait;
 static gint GdmFlexiReapDelayMinutes;
+static gint GdmBackgroundProgramInitialDelay = 30;
+static gint GdmBackgroundProgramRestartDelay = 30;
 
 static gboolean GdmAllowGtkThemeChange;
 static gboolean GdmTitleBar;
@@ -242,12 +245,12 @@ static gboolean GdmQuiver;
 static gboolean GdmShowGnomeFailsafe;
 static gboolean GdmShowXtermFailsafe;
 static gboolean GdmShowLastSession;
-static gboolean GdmUse24Clock;
 static gboolean GdmEntryCircles;
 static gboolean GdmEntryInvisible;
 static gboolean GdmGraphicalThemeRand;
 static gboolean GdmBroadcast;
 static gboolean GdmAllowAdd;
+static gboolean GdmRestartBackgroundProgram;
 
 /**
  * gdm_config_add_hash
@@ -405,7 +408,7 @@ gdm_config_init (void)
       &GdmShowXtermFailsafe, &bool_type);
    gdm_config_add_hash (GDM_KEY_SHOW_LAST_SESSION,
       &GdmShowLastSession, &bool_type);
-   gdm_config_add_hash (GDM_KEY_USE_24_CLOCK, &GdmUse24Clock, &bool_type);
+   gdm_config_add_hash (GDM_KEY_USE_24_CLOCK, &GdmUse24Clock, &string_type);
    gdm_config_add_hash (GDM_KEY_ENTRY_CIRCLES, &GdmEntryCircles, &bool_type);
    gdm_config_add_hash (GDM_KEY_ENTRY_INVISIBLE, &GdmEntryInvisible, &bool_type);
    gdm_config_add_hash (GDM_KEY_GRAPHICAL_THEME_RAND,
@@ -413,6 +416,8 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_BROADCAST, &GdmBroadcast, &bool_type);
    gdm_config_add_hash (GDM_KEY_ALLOW_ADD, &GdmAllowAdd, &bool_type);
    gdm_config_add_hash (GDM_KEY_SOUND_ON_LOGIN, &GdmSoundOnLogin, &bool_type);
+   gdm_config_add_hash (GDM_KEY_RESTART_BACKGROUND_PROGRAM,
+      &GdmRestartBackgroundProgram, &bool_type);
 
    /* string values */
    gdm_config_add_hash (GDM_KEY_PATH, &GdmPath, &string_type);
@@ -488,7 +493,7 @@ gdm_config_init (void)
       &GdmGraphicalThemeDir, &string_type);
    gdm_config_add_hash (GDM_KEY_GRAPHICAL_THEMES,
       &GdmGraphicalThemes, &string_type);
-   gdm_config_add_hash (GDM_KEY_GRAPHICAL_THEME_COLOR,
+   gdm_config_add_hash (GDM_KEY_GRAPHICAL_THEMED_COLOR,
       &GdmGraphicalThemeColor, &string_type);
    gdm_config_add_hash (GDM_KEY_INFO_MSG_FILE, &GdmInfoMsgFile, &string_type);
    gdm_config_add_hash (GDM_KEY_INFO_MSG_FONT, &GdmInfoMsgFont, &string_type);
@@ -523,6 +528,10 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_SCAN_TIME, &GdmScanTime, &int_type);
    gdm_config_add_hash (GDM_KEY_FLEXI_REAP_DELAY_MINUTES,
       &GdmFlexiReapDelayMinutes, &int_type);
+   gdm_config_add_hash (GDM_KEY_BACKGROUND_PROGRAM_INITIAL_DELAY,
+      &GdmBackgroundProgramInitialDelay, &int_type);
+   gdm_config_add_hash (GDM_KEY_BACKGROUND_PROGRAM_RESTART_DELAY,
+      &GdmBackgroundProgramRestartDelay, &int_type);
 }
 
 /**
@@ -531,7 +540,7 @@ gdm_config_init (void)
  * Get config file.  
  */
 static VeConfig *
-gdm_get_config (struct stat *statbuf)
+gdm_get_default_config (struct stat *statbuf)
 {
    int r;
 
@@ -544,13 +553,13 @@ gdm_get_config (struct stat *statbuf)
          return NULL;
       }
    } else {
-      VE_IGNORE_EINTR (r = g_stat (GDM_SYSCONFDIR_CONFIG_FILE, statbuf));
+      VE_IGNORE_EINTR (r = g_stat (GDM_DEFAULTS_CONF, statbuf));
       if (r < 0) {
          gdm_error (_("%s: No GDM configuration file: %s. Using defaults."),
-                      "gdm_config_parse", GDM_SYSCONFDIR_CONFIG_FILE);
+                      "gdm_config_parse", GDM_DEFAULTS_CONF);
          return NULL;
       } else {
-              config_file = GDM_SYSCONFDIR_CONFIG_FILE;
+              config_file = GDM_DEFAULTS_CONF;
       }
    }
 
@@ -567,12 +576,11 @@ static VeConfig *
 gdm_get_custom_config (struct stat *statbuf)
 {
    VeConfig *retval;
-   gchar *file = g_strdup_printf ("%s-custom", GDM_SYSCONFDIR_CONFIG_FILE);
    int r;
 
-   VE_IGNORE_EINTR (r = g_stat (file, statbuf));
+   VE_IGNORE_EINTR (r = g_stat (GDM_CUSTOM_CONF, statbuf));
    if (r >= 0) {
-      custom_config_file = file;
+      custom_config_file = g_strdup (GDM_CUSTOM_CONF);
       return ve_config_new (custom_config_file);
    } else {
       return NULL;
@@ -714,7 +722,8 @@ gdm_config_to_string (gchar *key, gchar **retval)
 /**
  * gdm_compare_displays
  * 
- * Support function for loading displays from the gdm.conf file
+ * Support function for loading displays from the configuration
+ * file
  */
 int
 gdm_compare_displays (gconstpointer a, gconstpointer b)
@@ -1714,7 +1723,7 @@ gdm_config_parse (void)
     * main configuration file.  If cfg is missing, then GDM will 
     * use the built-in defaults found in gdm.h.
     */
-   cfg                      = gdm_get_config (&statbuf);
+   cfg                      = gdm_get_default_config (&statbuf);
    config_file_mtime        = statbuf.st_mtime;
    custom_cfg               = gdm_get_custom_config (&statbuf);
    custom_config_file_mtime = statbuf.st_mtime;
