@@ -26,11 +26,11 @@
 #include <math.h>
 #include <gdk/gdkx.h>
 #include <locale.h>
-#include <libgnome/gnome-i18n.h>
-
-#include <vicious.h>
+#include <glib/gi18n.h>
 
 #include "gdmwm.h"
+#include "gdmcommon.h"
+#include "gdmconfig.h"
 #include "greeter_configuration.h"
 #include "greeter_parser.h"
 #include "greeter_events.h"
@@ -1083,31 +1083,32 @@ parse_stock (xmlNodePtr node,
       else if (g_ascii_strcasecmp ((char *) prop, "reboot") == 0)
         {
 	  g_free (*translated_text);
-	  *translated_text = g_strdup (_("_Reboot"));
+	  *translated_text = g_strdup (_("_Restart"));
 	}
       else if (g_ascii_strcasecmp ((char *) prop, "chooser") == 0)
         {
 	  g_free (*translated_text);
-	  *translated_text = g_strdup (_("_XDMCP Chooser"));
+	  *translated_text = g_strdup (_("Remote Login via _XDMCP"));
 	}
       else if (g_ascii_strcasecmp ((char *) prop, "config") == 0)
         {
 	  g_free (*translated_text);
-	  *translated_text = g_strdup (_("_Configure"));
+	  *translated_text = g_strdup (_("Confi_gure"));
+	}
+      else if (g_ascii_strcasecmp ((char *) prop, "options") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("Op_tions"));
 	}
       else if (g_ascii_strcasecmp ((char *) prop, "caps-lock-warning") == 0)
         {
 	  g_free (*translated_text);
-	  *translated_text = g_strdup (_("You've got capslock on!"));
+	  *translated_text = g_strdup (_("Caps Lock is on."));
 	}
       else if (g_ascii_strcasecmp ((char *) prop, "timed-label") == 0)
         {
 	  g_free (*translated_text);
-	  *translated_text = g_strdup (ngettext ("User %s will login in %d "
-				  		 "second",
-						 "User %s will login in %d "
-						 "seconds",
-						 1));
+	  *translated_text = g_strdup (_("User %u will login in %t"));
 	}
       else if (g_ascii_strcasecmp ((char *) prop, "welcome-label") == 0)
         {
@@ -1115,7 +1116,7 @@ parse_stock (xmlNodePtr node,
 	  welcome_string_info = info;
 
 	  g_free (*translated_text);
-	  *translated_text = g_strdup (GdmWelcome);
+	  *translated_text = gdm_common_get_welcomemsg ();
 	}
       /* FIXME: is this actually needed? */
       else if (g_ascii_strcasecmp ((char *) prop, "username-label") == 0)
